# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: docker_controler.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'docker_controler.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x64ocker_controler.proto\x12\x10\x64ocker_controler\"\x1d\n\x1bGetAllContainerNamesRequest\"7\n\x1cGetAllContainerNamesResponse\x12\x17\n\x0f\x63ontainer_names\x18\x01 \x03(\t\"%\n\x15StartContainerRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"2\n\x16StartContainerResponse\x12\x18\n\x10\x63ontainer_status\x18\x01 \x01(\t\"$\n\x14StopContainerRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"1\n\x15StopContainerResponse\x12\x18\n\x10\x63ontainer_status\x18\x01 \x01(\t\"\'\n\x17RestartContainerRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"4\n\x18RestartContainerResponse\x12\x18\n\x10\x63ontainer_status\x18\x01 \x01(\t2\xc9\x03\n\x16\x44ockerControlerService\x12w\n\x14GetAllContainerNames\x12-.docker_controler.GetAllContainerNamesRequest\x1a..docker_controler.GetAllContainerNamesResponse\"\x00\x12\x65\n\x0eStartContainer\x12\'.docker_controler.StartContainerRequest\x1a(.docker_controler.StartContainerResponse\"\x00\x12\x62\n\rStopContainer\x12&.docker_controler.StopContainerRequest\x1a\'.docker_controler.StopContainerResponse\"\x00\x12k\n\x10RestartContainer\x12).docker_controler.RestartContainerRequest\x1a*.docker_controler.RestartContainerResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'docker_controler_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_GETALLCONTAINERNAMESREQUEST']._serialized_start=44
  _globals['_GETALLCONTAINERNAMESREQUEST']._serialized_end=73
  _globals['_GETALLCONTAINERNAMESRESPONSE']._serialized_start=75
  _globals['_GETALLCONTAINERNAMESRESPONSE']._serialized_end=130
  _globals['_STARTCONTAINERREQUEST']._serialized_start=132
  _globals['_STARTCONTAINERREQUEST']._serialized_end=169
  _globals['_STARTCONTAINERRESPONSE']._serialized_start=171
  _globals['_STARTCONTAINERRESPONSE']._serialized_end=221
  _globals['_STOPCONTAINERREQUEST']._serialized_start=223
  _globals['_STOPCONTAINERREQUEST']._serialized_end=259
  _globals['_STOPCONTAINERRESPONSE']._serialized_start=261
  _globals['_STOPCONTAINERRESPONSE']._serialized_end=310
  _globals['_RESTARTCONTAINERREQUEST']._serialized_start=312
  _globals['_RESTARTCONTAINERREQUEST']._serialized_end=351
  _globals['_RESTARTCONTAINERRESPONSE']._serialized_start=353
  _globals['_RESTARTCONTAINERRESPONSE']._serialized_end=405
  _globals['_DOCKERCONTROLERSERVICE']._serialized_start=408
  _globals['_DOCKERCONTROLERSERVICE']._serialized_end=865
# @@protoc_insertion_point(module_scope)
